package adminkindboxhandler

import (
	"net/http"

	params "git.gocasts.ir/ebhomengo/niki/param"
	param "git.gocasts.ir/ebhomengo/niki/param/admin/kind_box"
	httpmsg "git.gocasts.ir/ebhomengo/niki/pkg/http_msg"
	"github.com/labstack/echo/v4"
)

// Enumerate godoc
// @Summary      Admin enumerate kindbox
// @Tags         Admins KindBoxes
// @Accept       json
// @Produce      json
// @Param 		 id path int true "KindBox ID"
// @Param 		 Request body param.EnumerateKindBoxRequest true "Request"
// @Success      204
// @Failure      400  {string}  "Bad Request"
// @Failure      401  {string}  "invalid or expired jwt"
// @Failure      403  {string}  "user not allowed"
// @Failure      422  {object}  httpmsg.ErrorResponse
// @Failure      500  {string}  "something went wrong"
// @Security 	 AuthBearerAdmin
// @Router       /admins/kindboxes/{id}/enumerate [patch].
func (h Handler) Enumerate(c echo.Context) error {
	var req param.EnumerateKindBoxRequest

	if bErr := c.Bind(&req); bErr != nil {
		return echo.NewHTTPError(http.StatusBadRequest)
	}

	resp, sErr := h.adminKindBoxSvc.Enumerate(c.Request().Context(), req)

	if sErr != nil {
		msg, code := httpmsg.Error(sErr)
		if resp.FieldErrors != nil {
			return c.JSON(code, echo.Map{
				"message": msg,
				"errors":  resp.FieldErrors,
			})
		}

		return echo.NewHTTPError(code, msg)
	}

	go h.notificationSvc.KindBoxEnumerated(params.NotificationKindBoxEnumerated{
		KindBoxID: req.KindBoxID,
	})

	return c.JSON(http.StatusNoContent, resp)
}
